pub enum BreakEternityError {
IterationFailedConverging {
z: f64,
},
ParseError {
parsed: String,
error: ParseFloatError,
},
LambertWError,
Arithmetic(ArithmeticError),
}Expand description
Error type for all errors in this crate.
Variants
IterationFailedConverging
An error that occurs when f_gamma and lambertw fails to converge a number (more than 100 iterations).
ParseError
An error that occurs when a String cannot be parsed to a Decimal.
Fields

error: ParseFloatErrorThe error that occurred.
LambertWError
An error that occurs when lambertw is called with a number less than -1.
Arithmetic(ArithmeticError)
An arithmetic error from a checked_* operation.